import { writable, readable } from "svelte/store";
import { Question, Exam, QuestionTypes } from "./data";

import CCNA2 from "../ccna2.json";
import CCNA3 from "../ccna3.json";
import CCNA4 from "../ccna4.json";
import TestData from "../data-test.json";

const Tests = new Map<string, Exam>();
Tests.set("ccna4", CCNA4);
Tests.set("ccna3", CCNA3);
Tests.set("ccna2", CCNA2);
Tests.set("test", TestData)

const DEFAULT = "ccna4";

export let test = new URL(window.location.href).searchParams.get("exam") || DEFAULT;

const dataVersion = Tests.has(test) ? test : DEFAULT;
const Data = Tests.get(dataVersion);
const QuestionMap = new Map<string, Question>();

Data.questions.forEach(q => QuestionMap.set(q.id, q));

console.log("Running exam:", dataVersion);

const runsShould = 3;

import Fuse from "fuse.js";

class QuestionManager {
    version: string = dataVersion;

    activeQuestion = writable<{
        correct: () => void;
        wrong: () => void;
        question: Question
    }>(undefined);


    progress = writable(0);

    private availableQuestions: {
        question: Question,
        level: number
    }[];

    index: Fuse<any, any>;

    private getAnswerString(question: Question) {
        switch (question.type) {
            case QuestionTypes.SelectOne:
            case QuestionTypes.SelectMultiple:
                return Object.values(question.options).join(" ; ");
            case QuestionTypes.TextInput:
                return question.correct;
            case QuestionTypes.AssignValues:
                return [...Object.values(question.fields), ...Object.values(question.values)].join(" ; ");
            default:
                return "";
        }
    }

    constructor() {
        this.availableQuestions = Data.questions.map(question => {
            return {
                question,
                level: Number(localStorage.getItem(question.id)) || 0
            }
        })

        this.getNewActive();

        const searchData = Data.questions
            .map(e => ({
                id: e.id,
                title: e.title,
                answers: this.getAnswerString(e)
            }))

        this.index = new Fuse(searchData, {
            keys: [{
                name: "title",
                weight: 0.8
            }, {
                name: "answers",
                weight: 0.2
            }]
        })
    }

    search(term: string) {
        if (term === "")
            return Data.questions;
        const match = (this.index.search(term) as any[]).map(e => QuestionMap.get(e.id));
        return match;
    }

    getNewActive() {
        let progress = 0;
        let highest = -1;
        this.availableQuestions.forEach(question => {
            const runs = question.level;
            if (runs > highest)
                highest = runs;

            progress += runs > runsShould ? runsShould : runs;
        })

        this.progress.set((progress / (this.availableQuestions.length * runsShould)) * 100);

        let options = this.availableQuestions.filter(question => question.level < highest);
        if (options.length <= 0)
            options = this.availableQuestions;

        let idx = Math.floor(Math.random() * options.length);
        let randomItem = options[idx];
        console.log("Setting question", randomItem);
        this.activeQuestion.set({
            question: randomItem.question,
            correct: () => {
                randomItem.level++;
                localStorage.setItem(randomItem.question.id, randomItem.level.toString());
                this.getNewActive();
            },
            wrong: () => {
                this.getNewActive();
            }
        })
    }
}

export default new QuestionManager();

import Theme from "./helper/theme";

export enum DeviceTypes {
    MOBILE,
    DESKTOP
}

const deviceTypeQuery = window.matchMedia("(min-width: 60rem)");

export const DeviceType = writable<DeviceTypes>(Number(deviceTypeQuery.matches));

const onChange = (ev: MediaQueryListEvent) => DeviceType.set(Number(ev.matches));
deviceTypeQuery.addListener(onChange);

export const IsDark = readable(Theme.isDark(), set => {
    set(Theme.isDark());
    const onChange = (val) => set(val);
    Theme.isDarkObservable.subscribe(onChange)
    return () => Theme.isDarkObservable.unsubscribe(onChange);
})

export { ModalStore } from "./modals";

export const AvailableTests = Array.from(Tests.keys());